/* -----------------------------------------------------------------------
 *
 * Project:     ttsnmp
 *
 * Filename:    ttCallList.c
 *
 * Date:        March 1995
 *
 * Purpose:     Call list implementation
 *
 * Author:      Bram van der Waaij      
 *
 *           _o______=====____H_=======_H______=====__        __=====_____H_===
 *          |_______|[[[[[     \#######/       [[[[[ o|      |o ]]]]]      \###
 *     ______| [][] |____    B.D. van der Waaij      ||      || 
 *    |o ____|      |__| \___________________________||      ||________________
 *  |_| /DCC |_7189_|   \_|  |  |  |  |  |  |  |  |  ||_|  |_||  |  |  |  |  | 
 * @|_|=|====|======|=o=|=============================|_|@@|_|=|====|======|===
 *  |_| [(o)==(o)]  |__________________|   [(o)==(o)] |_|\/|_| [(o)==(o)]  |___
 * ============================================================================
 ***[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]****[]*/


#include <UT-SNMP.h>
#include <ttCallList.h>


typedef struct TCallListElem {
  int                   CallID;	/* calling identifier */
  TCallInfo             info;	/* agent info struct */
  struct TCallListElem *next;	/* pointer to next agent in list*/
} TCallListElem;

static TCallListElem *rootCallList; /* root pointer to Calling list */


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	CreateCallList
 *
 * Arguments:	None
 *
 * Return:	OK
 *
 * Purpose:     Reset CallList root pointer;
 *
 * Remarks:	CallList must be empty
 *
 * -----------------------------------------------------------------------
 */
STATUS CreateCallList()
{
  rootCallList = NULL;

  return OK;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	RemoveCallList
 *
 * Arguments:	None
 *
 * Return:	OK
 *
 * Purpose:     Remove all Calls from CallList
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS RemoveCallList()
{
  TCallListElem *check, *p;

  check = rootCallList;
  p     = rootCallList;

  while (check) {     
    check = p->next;
    free(p);
    p = check;
  }  

  rootCallList = NULL;
  return OK;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AddCall
 *
 * Arguments:	callid, the call number (IN)
 *		CallInfo, the call information (IN)   
 *
 * Return:	OK, FAILED if no memory
 *
 * Purpose:     Add a call to the CallList
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
STATUS AddCall( int callid, TCallInfo CallInfo )
{
  TCallListElem *newElem, *p;

    /* create new element */
  newElem = (struct TCallListElem *) malloc( sizeof( struct TCallListElem ) );    
  if (newElem == NULL)
    return FAILED;

  newElem->CallID = callid;
  strcpy(newElem->info.agentName,   CallInfo.agentName);
  strcpy(newElem->info.tclcallback, CallInfo.tclcallback); 
  newElem->info.timeout = CallInfo.timeout;

  if (!rootCallList)   /* first element in list */
  {
    newElem->next = NULL;
    rootCallList = newElem;
    return OK;    
  }

    /* search for last item */
  p = rootCallList;
  while (p->next) {
    p = p->next;
  }

    /* add newElem to list */
  newElem->next = NULL;
  p->next = newElem;
  return OK;  
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	DelCall
 *
 * Arguments:	callid, the call number (IN)
 *
 * Return:	OK      if callid is in the CallList
 *		FAILED  if callid is not in the CallList
 *
 * Purpose:     Remove a call from the CallList
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS DelCall( int callid )
{
  TCallListElem *last, *p;

  p    = rootCallList;
  last = NULL;

  while (p) {
    if (p->CallID == callid) {  	/* found ? */

      if (last == NULL) {              /* the first in the list */
        rootCallList = p->next;        /* new root pointer */  
        free(p);
      }         
      else {                          /* somewhere in the list */
        last->next = p->next;         /* skip this Element */
        free(p);
      }

      return OK;
    }
    last = p;
    p = p->next;
  }

  return FAILED;  /* callid not found */
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	GetCall
 *
 * Arguments:	callid, the call number (IN
 *		CallInfo  the call information (OUT)
 *
 * Return:	OK     if callid is found
 *		FAILED if callid is not found
 *
 * Purpose:     retrieve call information from callid
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
STATUS GetCall( int callid,  TCallInfo *CallInfo )
{
  TCallListElem *p;

  p = rootCallList;

  while (p) {
    if (p->CallID == callid) {    /* found ? */
      *CallInfo = p->info;
      return OK;
    }
    p = p->next;
  }

  return FAILED;
}


/*
 * -----------------------------------------------------------------------
 *
 * Function:	ForEachCall
 *
 * Arguments:	*FECall, pointer to user proc
 *
 * Return:	OK
 *
 * Purpose:     Each item in the list is given to FECall
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
STATUS ForEachCall( ForEachCallProc *FECall )
{
  TCallListElem *p;

  p = rootCallList;

  while (p) {
    FECall (p->CallID, &(p->info));
    p = p->next;
  }

  return OK;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintCallList
 *
 * Arguments:	none
 *
 * Return:	OK
 *
 * Purpose:     print the CallList
 *
 * Remarks:	only for test purposes
 *
 * -----------------------------------------------------------------------
 */
STATUS PrintCallList()
{
  TCallListElem *p;
  AsnOcts *ipaddr;

  p = rootCallList;

  if (p == NULL)
    fprintf(stdout,"no calls waiting\n");
  else
    while (p) {
       fprintf(stdout,"%i: %s to %s  - %isec\n",
                       p->CallID, p->info.agentName, p->info.tclcallback,
                       p->info.timeout);
      p = p->next;
    }

  return OK;
}
